from .admin import (
    CardInline,
    ChargeScheduleInline,
    ChargeInline,
    CustomerAdmin,
    CardAdmin,
    RefundAdmin,
    EventAdmin,
    RefundInline,
    ChargeAdmin,
    SourceAdmin,
    OccurrenceInline,
    ScheduleAdmin,
    OccurrenceAdmin,
    ChargeScheduleAdmin,
)



from django.contrib.admin.sites import AdminSite
from django.contrib.auth import get_user_model

from django_omise.admin import ChargeAdmin, ChargeInline
from django_omise.models.core import Charge, Source, Card
from django_omise.models.choices import Currency, ChargeStatus

from django_omise.tests.base import ClientAndUserBaseTestCase, OmiseBaseTestCase
from django_omise.tests.mock_django import MockRequest

from unittest import mock


User = get_user_model()


class AdminChargeTestCase(ClientAndUserBaseTestCase, OmiseBaseTestCase):
    def test_charge_chage_permission(self):
        charge_admin = ChargeAdmin(model=Charge, admin_site=AdminSite())
        request = MockRequest(user=self.user)
        self.assertEqual(charge_admin.has_change_permission(request=request), False)

    @mock.patch("django_omise.admin.admin.format_html")
    def test_charge_colorized_status_successful(self, mock_format_html):
        charge_admin = ChargeAdmin(model=Charge, admin_site=AdminSite())

        charge = self.create_charge(status=ChargeStatus.SUCCESSFUL)
        charge_admin.colorized_status(obj=charge)
        mock_format_html.assert_called_once()

    @mock.patch("django_omise.admin.admin.format_html")
    def test_charge_colorized_status_pending(self, mock_format_html):
        charge_admin = ChargeAdmin(model=Charge, admin_site=AdminSite())

        charge = self.create_charge(status=ChargeStatus.PENDING)
        charge_admin.colorized_status(obj=charge)
        mock_format_html.assert_called_once()

    @mock.patch("django_omise.admin.admin.format_html")
    def test_charge_colorized_status_failed(self, mock_format_html):
        charge_admin = ChargeAdmin(model=Charge, admin_site=AdminSite())

        charge = self.create_charge(status=ChargeStatus.FAILED)
        charge_admin.colorized_status(obj=charge)
        mock_format_html.assert_called_once()

    @mock.patch("django_omise.admin.admin.format_html")
    def test_charge_colorized_status_partially_refunded(self, mock_format_html):
        charge_admin = ChargeAdmin(model=Charge, admin_site=AdminSite())

        charge = self.create_charge(
            status=ChargeStatus.SUCCESSFUL,
            refunded_amount=10000,
        )
        charge_admin.colorized_status(obj=charge)
        mock_format_html.assert_called_once()

    @mock.patch("django_omise.admin.admin.format_html")
    def test_charge_inline_colorized_status_successful(self, mock_format_html):
        charge_inline = ChargeInline(parent_model=Charge, admin_site=AdminSite())

        charge = self.create_charge(
            status=ChargeStatus.SUCCESSFUL,
        )
        charge_inline.colorized_status(obj=charge)
        mock_format_html.assert_called_once()

    @mock.patch("django_omise.admin.admin.format_html")
    def test_charge_inline_colorized_status_failed(self, mock_format_html):
        charge_inline = ChargeInline(parent_model=Charge, admin_site=AdminSite())

        charge = self.create_charge(
            status=ChargeStatus.FAILED,
        )
        charge_inline.colorized_status(obj=charge)
        mock_format_html.assert_called_once()

    @mock.patch("django_omise.admin.admin.format_html")
    def test_charge_inline_colorized_status_pending(self, mock_format_html):
        charge_inline = ChargeInline(parent_model=Charge, admin_site=AdminSite())

        charge = self.create_charge(
            status=ChargeStatus.PENDING,
        )
        charge_inline.colorized_status(obj=charge)
        mock_format_html.assert_called_once()

    @mock.patch("django_omise.admin.admin.format_html")
    def test_charge_inline_colorized_status_refunded(self, mock_format_html):
        charge_inline = ChargeInline(parent_model=Charge, admin_site=AdminSite())

        charge = self.create_charge(
            status=ChargeStatus.SUCCESSFUL,
            refunded_amount=1000,
        )
        charge_inline.colorized_status(obj=charge)
        mock_format_html.assert_called_once()

    def test_charge_inline_change_permission(self):
        charge_inline = ChargeInline(parent_model=Charge, admin_site=AdminSite())
        request = MockRequest(user=self.user)

        self.assertEqual(charge_inline.has_change_permission(request=request), False)

    def test_charge_source_type_with_card(self):
        charge_admin = ChargeAdmin(model=Charge, admin_site=AdminSite())

        card = self.create_card()
        charge = self.create_charge(status=ChargeStatus.SUCCESSFUL, card=card)
        self.assertIn("card", charge_admin.source_type(obj=charge))

    def test_charge_source_type_with_source(self):
        charge_admin = ChargeAdmin(model=Charge, admin_site=AdminSite())

        source = self.create_source()
        charge = self.create_charge(status=ChargeStatus.SUCCESSFUL, source=source)
        self.assertNotIn("card", charge_admin.source_type(obj=charge))



token_response = """{
  "object": "token",
  "id": "test_token_id",
  "livemode": false,
  "location": "https://vault.omise.co/tokens/tokn_test_5s1k1g5opc8lvwva73y",
  "used": false,
  "charge_status": "unknown",
  "card": {
    "object": "card",
    "id": "card_test_5s1k1g5nl9it6p44gda",
    "livemode": false,
    "location": null,
    "deleted": false,
    "street1": null,
    "street2": null,
    "city": "Bangkok",
    "state": null,
    "phone_number": null,
    "postal_code": "10320",
    "country": "us",
    "financing": "credit",
    "bank": "JPMORGAN CHASE BANK N.A.",
    "brand": "Visa",
    "fingerprint": "7jGh9VWw5Dvm/8Zrh9wY/e3VXhdWyg8k8xD1+8sHGhU=",
    "first_digits": null,
    "last_digits": "4242",
    "name": "JOHN DOE",
    "expiration_month": 6,
    "expiration_year": 2024,
    "security_code_check": true,
    "tokenization_method": null,
    "created_at": "2022-06-05T09:44:09Z"
  },
  "created_at": "2022-06-05T09:44:09Z"
}"""



from django_omise.models.event import EventType, Event
from django_omise.models.core import Charge
from django_omise.omise import omise

from django_omise.utils.core_utils import update_or_create_from_omise_object


from typing import Dict


def pre_event_handle(omise_event: omise.Event, raw_event: Dict):
    """
    Perform additional actions on Omise Event object before handling with the webhook view.

    :params omise_event: The Omise Event object
    :param raw_event: The event data as received with webhook view

    :returns: None
    """
    pass


def post_event_handle(omise_event: omise.Event, event_object: Event, raw_event: Dict):
    """
    Perform additional actions on Omise Event object after handling with the webhook view.

    :params omise_event: The Omise Event object
    :param event_object: The new event object in the database
    :param raw_event: The event data as received with webhook view

    :returns: None
    """
    pass



from django.test import TestCase

from django.contrib.auth import get_user_model

from django_omise.models.core import Customer
from django_omise.models.schedule import Schedule, ChargeSchedule
from django_omise.models.choices import Currency, SchedulePeriod, ScheduleStatus


from django.utils import timezone

from unittest import mock

from .test_utils import (
    mocked_requests_post,
    mocked_requests_get,
    mocked_add_card_request,
    mocked_delete_card_request,
)

User = get_user_model()

# Create your tests here.
class CustomerTestCase(TestCase):
    def setUp(self):
        self.user = User.objects.create_user(
            username="test_user1", email="test_user1@email.com"
        )

        self.post_patcher = mock.patch(
            "requests.post", side_effect=mocked_requests_post
        )
        self.get_patcher = mock.patch("requests.get", side_effect=mocked_requests_get)

        self.post_patcher.start()
        self.get_patcher.start()

        self.customer, self.created = Customer.get_or_create(user=self.user)
        self.customer.sync_cards()

    def tearDown(self):
        mock.patch.stopall()

    def test_create_customer(self):
        self.assertTrue(self.created)

        self.customer, created = Customer.get_or_create(user=self.user)
        self.assertFalse(created)

        self.assertEqual(self.customer.user, self.user)
        self.assertEqual(self.customer.id, self.customer.get_omise_object().id)

    def test_create_customer_wrong_mode(self):
        with self.settings(OMISE_LIVE_MODE=True), self.assertRaises(ValueError):
            customer, created = Customer.get_or_create(user=self.user)

    def test_sync_customer_cards(self):
        omise_customer = self.customer.get_omise_object()

        self.assertEqual(self.customer.cards.count(), len(omise_customer.cards))

    def test_customer_str_with_user(self):
        self.assertIn(str(self.user), str(self.customer))

    def test_customer_str_without_user(self):
        customer = Customer.objects.create(id="customer_test_id", livemode=False)
        self.assertIn("customer_test_id", str(customer))

    @mock.patch("requests.patch", side_effect=mocked_add_card_request)
    def test_add_card_to_customer(self, mocked_request):
        initial_card_count = self.customer.cards.count()
        self.customer.add_card(token="test_token_id")
        self.assertEqual(initial_card_count + 1, self.customer.cards.count())

    @mock.patch("requests.delete", side_effect=mocked_delete_card_request)
    def test_delete_card(self, mock_request):
        live_cards_count = self.customer.cards.live().count()

        self.customer.remove_card(card=self.customer.cards.first())

        self.assertEqual(live_cards_count - 1, self.customer.cards.live().count())

    @mock.patch("django_omise.models.core.Charge.charge")
    def test_charge_with_card(self, mock_charge):
        self.customer.charge_with_card(
            amount=100000,
            currency=Currency.THB,
            card=self.customer.cards.live().first(),
        )

        mock_charge.assert_called_once()

        args, kwargs = mock_charge.call_args
        self.assertEqual(kwargs["card"], self.customer.cards.live().first())

    def test_schedules(self):
        charge_schedule = ChargeSchedule.objects.create(
            id="test_charge_schedule_id",
            amount=100000,
            currency=Currency.THB,
            livemode=False,
            card=self.customer.cards.live().first(),
            customer=self.customer,
            default_card=False,
        )

        schedule = Schedule.objects.create(
            id="test_schedule_id",
            livemode=False,
            active=True,
            charge=charge_schedule,
            end_on=timezone.now().date(),
            ended_at=timezone.now(),
            every=1,
            period=SchedulePeriod.DAY,
            start_on=timezone.now().date(),
            status=ScheduleStatus.RUNNING,
        )

        self.assertIn(schedule, self.customer.schedules.all())

    @mock.patch(
        "django_omise.models.schedule.Schedule.update_or_create_from_omise_object"
    )
    @mock.patch("django_omise.models.core.omise.Schedule.create")
    def test_create_schedule_without_card(
        self, mock_schedule_create, mock_update_or_create_method
    ):
        self.customer.create_schedule(
            amount=100000,
            currency=Currency.THB,
            every=1,
            period=SchedulePeriod.DAY,
            start_date=timezone.now(),
            end_date=timezone.now(),
        )

        args, kwargs = mock_schedule_create.call_args
        charge_schedule = kwargs["charge"]
        self.assertNotIn("card", charge_schedule)

    @mock.patch(
        "django_omise.models.schedule.Schedule.update_or_create_from_omise_object"
    )
    @mock.patch("django_omise.models.core.omise.Schedule.create")
    def test_create_schedule_with_card(
        self, mock_schedule_create, mock_update_or_create_method
    ):
        self.customer.create_schedule(
            amount=100000,
            currency=Currency.THB,
            card=self.customer.cards.live().first(),
            every=1,
            period=SchedulePeriod.DAY,
            start_date=timezone.now(),
            end_date=timezone.now(),
        )

        args, kwargs = mock_schedule_create.call_args
        charge_schedule = kwargs["charge"]
        self.assertIn("card", charge_schedule)

    @mock.patch(
        "django_omise.models.schedule.Schedule.update_or_create_from_omise_object"
    )
    @mock.patch("django_omise.models.core.omise.Schedule.create")
    def test_create_schedule_without_description(
        self, mock_schedule_create, mock_update_or_create_method
    ):
        self.customer.create_schedule(
            amount=100000,
            currency=Currency.THB,
            card=self.customer.cards.live().first(),
            every=1,
            period=SchedulePeriod.DAY,
            start_date=timezone.now(),
            end_date=timezone.now(),
        )

        args, kwargs = mock_schedule_create.call_args
        charge_schedule = kwargs["charge"]
        self.assertNotIn("description", charge_schedule)

    @mock.patch(
        "django_omise.models.schedule.Schedule.update_or_create_from_omise_object"
    )
    @mock.patch("django_omise.models.core.omise.Schedule.create")
    def test_create_schedule_with_description(
        self, mock_schedule_create, mock_update_or_create_method
    ):
        description = "Test description"
        self.customer.create_schedule(
            amount=100000,
            currency=Currency.THB,
            card=self.customer.cards.live().first(),
            description=description,
            every=1,
            period=SchedulePeriod.DAY,
            start_date=timezone.now(),
            end_date=timezone.now(),
        )

        args, kwargs = mock_schedule_create.call_args
        charge_schedule = kwargs["charge"]
        self.assertIn("description", charge_schedule)



customer_response = """{
  "object": "customer",
  "id": "cust_test_5s1jz157366mu6wr0ng",
  "livemode": false,
  "location": "/customers/cust_test_5s1jz157366mu6wr0ng",
  "deleted": false,
  "metadata": {
  },
  "cards": {
    "object": "list",
    "data": [
      {
        "object": "card",
        "id": "card_test_5s1jzgw7oda499o8k0y",
        "livemode": false,
        "location": "/customers/cust_test_5s1jz157366mu6wr0ng/cards/card_test_5s1jzgw7oda499o8k0y",
        "deleted": false,
        "street1": null,
        "street2": null,
        "city": "Bangkok",
        "state": null,
        "phone_number": null,
        "postal_code": "10320",
        "country": "us",
        "financing": "credit",
        "bank": "JPMORGAN CHASE BANK N.A.",
        "brand": "Visa",
        "fingerprint": "7jGh9VWw5Dvm/8Zrh9wY/e3VXhdWyg8k8xD1+8sHGhU=",
        "first_digits": null,
        "last_digits": "4242",
        "name": "JOHN DOE",
        "expiration_month": 6,
        "expiration_year": 2024,
        "security_code_check": true,
        "tokenization_method": null,
        "created_at": "2022-06-05T09:38:32Z"
      },
      {
        "object": "card",
        "id": "card_test_5s1jzsw6iy9nw4gzmx3",
        "livemode": false,
        "location": "/customers/cust_test_5s1jz157366mu6wr0ng/cards/card_test_5s1jzsw6iy9nw4gzmx3",
        "deleted": false,
        "street1": null,
        "street2": null,
        "city": "Bangkok",
        "state": null,
        "phone_number": null,
        "postal_code": "10320",
        "country": "us",
        "financing": "credit",
        "bank": "JPMORGAN CHASE BANK N.A.",
        "brand": "Visa",
        "fingerprint": "7jGh9VWw5Dvm/8Zrh9wY/e3VXhdWyg8k8xD1+8sHGhU=",
        "first_digits": null,
        "last_digits": "4242",
        "name": "JOHN DOE",
        "expiration_month": 6,
        "expiration_year": 2024,
        "security_code_check": true,
        "tokenization_method": null,
        "created_at": "2022-06-05T09:39:29Z"
      }
    ],
    "limit": 20,
    "offset": 0,
    "total": 2,
    "location": "/customers/cust_test_5s1jz157366mu6wr0ng/cards",
    "order": "chronological",
    "from": "1970-01-01T00:00:00Z",
    "to": "2022-06-05T09:39:36Z"
  },
  "default_card": "card_test_5s1jzgw7oda499o8k0y",
  "description": "John Doe (id: 30)",
  "email": "john.doe@example.com",
  "created_at": "2022-06-05T09:37:17Z"
}"""


add_card_response = """{
  "object": "customer",
  "id": "cust_test_5s1jz157366mu6wr0ng",
  "livemode": false,
  "location": "/customers/cust_test_5s1jz157366mu6wr0ng",
  "deleted": false,
  "metadata": {
  },
  "cards": {
    "object": "list",
    "data": [
      {
        "object": "card",
        "id": "card_test_5s1jzgw7oda499o8k0y",
        "livemode": false,
        "location": "/customers/cust_test_5s1jz157366mu6wr0ng/cards/card_test_5s1jzgw7oda499o8k0y",
        "deleted": false,
        "street1": null,
        "street2": null,
        "city": "Bangkok",
        "state": null,
        "phone_number": null,
        "postal_code": "10320",
        "country": "us",
        "financing": "credit",
        "bank": "JPMORGAN CHASE BANK N.A.",
        "brand": "Visa",
        "fingerprint": "7jGh9VWw5Dvm/8Zrh9wY/e3VXhdWyg8k8xD1+8sHGhU=",
        "first_digits": null,
        "last_digits": "4242",
        "name": "JOHN DOE",
        "expiration_month": 6,
        "expiration_year": 2024,
        "security_code_check": true,
        "tokenization_method": null,
        "created_at": "2022-06-05T09:38:32Z"
      },
      {
        "object": "card",
        "id": "card_test_5s1jzsw6iy9nw4gzmx3",
        "livemode": false,
        "location": "/customers/cust_test_5s1jz157366mu6wr0ng/cards/card_test_5s1jzsw6iy9nw4gzmx3",
        "deleted": false,
        "street1": null,
        "street2": null,
        "city": "Bangkok",
        "state": null,
        "phone_number": null,
        "postal_code": "10320",
        "country": "us",
        "financing": "credit",
        "bank": "JPMORGAN CHASE BANK N.A.",
        "brand": "Visa",
        "fingerprint": "7jGh9VWw5Dvm/8Zrh9wY/e3VXhdWyg8k8xD1+8sHGhU=",
        "first_digits": null,
        "last_digits": "4242",
        "name": "JOHN DOE",
        "expiration_month": 6,
        "expiration_year": 2024,
        "security_code_check": true,
        "tokenization_method": null,
        "created_at": "2022-06-05T09:39:29Z"
      },
      {
        "object": "card",
        "id": "card_test_5s1k1g5nl9it6p44gda",
        "livemode": false,
        "location": "/customers/cust_test_5s1jz157366mu6wr0ng/cards/card_test_5s1k1g5nl9it6p44gda",
        "deleted": false,
        "street1": null,
        "street2": null,
        "city": "Bangkok",
        "state": null,
        "phone_number": null,
        "postal_code": "10320",
        "country": "us",
        "financing": "credit",
        "bank": "JPMORGAN CHASE BANK N.A.",
        "brand": "Visa",
        "fingerprint": "7jGh9VWw5Dvm/8Zrh9wY/e3VXhdWyg8k8xD1+8sHGhU=",
        "first_digits": null,
        "last_digits": "4242",
        "name": "JOHN DOE",
        "expiration_month": 6,
        "expiration_year": 2024,
        "security_code_check": true,
        "tokenization_method": null,
        "created_at": "2022-06-05T09:59:06Z"
      }
    ],
    "limit": 20,
    "offset": 0,
    "total": 3,
    "location": "/customers/cust_test_5s1jz157366mu6wr0ng/cards",
    "order": "chronological",
    "from": "1970-01-01T00:00:00Z",
    "to": "2022-06-05T09:59:14Z"
  },
  "default_card": "card_test_5s1jzgw7oda499o8k0y",
  "description": "John Doe (id: 30)",
  "email": "john.doe@example.com",
  "created_at": "2022-06-05T09:37:17Z"
}"""



delete_card_response = """{
  "object": "card",
  "id": "card_test_5s1jzsw6iy9nw4gzmx3",
  "livemode": false,
  "location": "/customers/cust_test_5s1jz157366mu6wr0ng/cards/card_test_5s1jzsw6iy9nw4gzmx3",
  "deleted": true,
  "street1": null,
  "street2": null,
  "city": "Bangkok",
  "state": null,
  "phone_number": null,
  "postal_code": "10320",
  "country": "us",
  "financing": "credit",
  "bank": "JPMORGAN CHASE BANK N.A.",
  "brand": "Visa",
  "fingerprint": "7jGh9VWw5Dvm/8Zrh9wY/e3VXhdWyg8k8xD1+8sHGhU=",
  "first_digits": null,
  "last_digits": "4242",
  "name": "JOHN DOE",
  "expiration_month": null,
  "expiration_year": null,
  "security_code_check": true,
  "tokenization_method": null,
  "created_at": "2022-06-05T09:39:29Z"
}"""



from django.test import TestCase

from django_omise.models.core import Refund
from django_omise.models.choices import Currency

# Create your tests here.
class RefundTestCase(TestCase):
    def test_refund_human_amount(self):
        refund = Refund(amount=100000, currency=Currency.THB)
        self.assertEqual(refund.human_amount, "1,000.00")

    def test_refund_human_amount_jpy(self):
        refund = Refund(amount=100000, currency=Currency.JPY)
        self.assertEqual(refund.human_amount, "100,000.00")



from django.contrib.auth import get_user_model

from django.test import Client, TestCase

from django_omise.models.core import Customer, Card, Charge, Source
from django_omise.models.choices import ChargeStatus, Currency

User = get_user_model()


class ClientAndUserBaseTestCase(TestCase):
    """Base TestCase with utilites to create user and login client."""

    def setUp(self):
        """Class setup."""
        self.client = Client()
        self.index_url = "/"
        self.login()

    def create_user(self):
        """Create user and returns username, password tuple."""
        username, password = "admin", "test"
        user = User.objects.get_or_create(
            username=username,
            email="admin@test.com",
            is_superuser=True,
            is_staff=True,
        )[0]
        user.set_password(password)
        user.save()
        self.user = user
        return (username, password)

    def login(self):
        """Log in client session."""
        username, password = self.create_user()
        self.client.login(username=username, password=password)


class OmiseBaseTestCase(TestCase):
    def create_card(self, **kwargs):
        default = {
            "id": "test_card_id",
            "livemode": False,
        }

        default.update(kwargs)

        return Card.objects.create(**default)

    def create_source(self, **kwargs):
        default = {
            "id": "test_source_id",
            "livemode": True,
            "amount": 100000,
            "currency": Currency.THB,
        }

        default.update(kwargs)
        return Source.objects.create(**default)

    def create_customer(self, **kwargs):
        default = {
            "id": "test_customer_id",
            "livemode": False,
        }
        default.update(kwargs)
        return Customer.objects.create(**default)

    def create_charge(self, **kwargs):
        default = {
            "id": "test_charge_id",
            "livemode": False,
            "status": ChargeStatus.SUCCESSFUL,
            "amount": 1000000,
            "currency": Currency.THB,
            "fee": 0,
            "fee_vat": 0,
            "net": 0,
            "paid": True,
            "refundable": True,
            "refunded_amount": 0,
        }
        default.update(kwargs)
        charge = Charge.objects.create(
            **default,
        )
        return charge



DEBUG = True
ROOT_URLCONF = "urls"

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.postgresql",
        "NAME": "django_omise",
        "USER": "django_omise",
        "PASSWORD": "django_omise",
        "HOST": "localhost",
        "PORT": "5431",
    }
}

# DATABASES = {"default": {"ENGINE": "django.db.backends.sqlite3"}}

USE_TZ = True
TIME_ZONE = "Asia/Bangkok"

INSTALLED_APPS = (
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.admin",
    "django_omise",
)

SECRET_KEY = "django_tests_secret_key"

OMISE_PUBLIC_KEY = "test_omise_public_key"
OMISE_SECRET_KEY = "test_omise_secret_key"
OMISE_LIVE_MODE = False

MIDDLEWARE = (
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
)

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]



from django.contrib.auth.mixins import UserPassesTestMixin
from django.contrib import messages

from django.urls import reverse
from django.views.generic import FormView

from django_omise.forms import IssueRefundForm
from django_omise.models.core import Charge
from django_omise.omise import omise


class IssueRefundView(UserPassesTestMixin, FormView):
    template_name = "admin/django_omise/charge/issue_refund_form.html"
    form_class = IssueRefundForm

    def test_func(self):
        return self.request.user.has_perm("django_omise.issue_refund")

    def get_form_kwargs(self, *args, **kwargs):
        kwargs = super().get_form_kwargs(*args, **kwargs)
        kwargs["charge"] = self.get_object()
        return kwargs

    def get_object(self):
        return Charge.objects.get(id=self.kwargs["pk"])

    def get_context_data(self, *args, **kwargs):
        context = super().get_context_data(*args, **kwargs)
        context["charge"] = self.get_object()
        return context

    def get_success_url(self):
        return reverse(
            "admin:django_omise_charge_change",
            args=[
                self.get_object().pk,
            ],
        )

    def form_valid(self, form):
        refund_amount = form.cleaned_data["refund_amount"]
        charge = self.get_object()
        omise_charge = charge.get_omise_object()

        try:
            omise_charge.refund(amount=refund_amount)
        except omise.errors.FailedRefundError as e:
            messages.error(self.request, str(e))
            return self.form_invalid(form)

        messages.success(
            self.request,
            "Refund issued. This might take a few seconds to update. Try refreshing if you do not see the refund data.",
        )
        return super().form_valid(form)



import json

from .mockdata.card import delete_card_response
from .mockdata.charge import (
    base_charge_response,
    base_charge_with_metadata_response,
    base_charge_jpy,
    partially_refunded_response,
    fully_refunded_response,
    charge_with_schedule_response,
)
from .mockdata.customer import customer_response, add_card_response
from .mockdata.event import schedule_with_one_charge_event_response
from .mockdata.schedule import (
    base_schedule_response,
    schedule_response_no_next_occurrences_on,
)
from .mockdata.token import token_response


class MockResponse:
    def __init__(self, raw_response, status_code):
        self.status_code = status_code
        self.raw_response = raw_response

    def json(self):
        return json.loads(self.raw_response)


def mocked_requests_post(*args, **kwargs):

    request_url = args[0]

    if request_url == "https://api.omise.co/customers":  # omise.Customer.create
        return MockResponse(customer_response, 200)

    if request_url == "https://vault.omise.co/tokens":  # omise.Token.create
        return MockResponse(customer_response, 200)

    return MockResponse(None, 404)


def mocked_requests_get(*args, **kwargs):

    request_url = args[0]

    if "https://api.omise.co/customers" in request_url:  # omise.Customer.retrieve
        return MockResponse(customer_response, 200)

    if (
        request_url == "https://vault.omise.co/tokens/test_token_id"
    ):  # omise.Token.retrieve
        return MockResponse(token_response, 200)

    return MockResponse(None, 404)


def mocked_base_customer_request(*args, **kwargs):
    return MockResponse(customer_response, 200)


def mocked_base_charge_request(*args, **kwargs):
    return MockResponse(base_charge_response, 200)


def mocked_set_charge_metadata_request(*args, **kwargs):
    return MockResponse(base_charge_with_metadata_response, 200)


def mocked_charge_with_card_request(*args, **kwargs):
    return MockResponse(base_charge_response, 200)


def mocked_charge_with_schedule_request(*args, **kwargs):
    return MockResponse(charge_with_schedule_response, 200)


def mocked_partially_refunded_charge_request(*args, **kwargs):
    return MockResponse(partially_refunded_response, 200)


def mocked_fully_refunded_charge_request(*args, **kwargs):
    return MockResponse(fully_refunded_response, 200)


def mocked_jpy_charge(*args, **kwargs):
    return MockResponse(base_charge_jpy, 200)


def mocked_add_card_request(*args, **kwargs):
    return MockResponse(add_card_response, 200)


def mocked_delete_card_request(*args, **kwargs):
    return MockResponse(delete_card_response, 200)


def mocked_base_schedule_request(*args, **kwargs):
    return MockResponse(base_schedule_response, 200)


def mocked_schedule_without_next_occurrences_on_request(*args, **kwargs):
    return MockResponse(schedule_with_one_charge_event_response, 200)


def mocked_schedule_event_request(*args, **kwargs):
    return MockResponse(schedule_with_one_charge_event_response, 200)


def mocked_requests_event_schedule(*args, **kwargs):

    request_url = args[0]

    if "https://api.omise.co/events" in request_url:  # omise.Event.retrieve
        return MockResponse(schedule_with_one_charge_event_response, 200)

    if "https://api.omise.co/schedules" in request_url:  # omise.Event.retrieve
        return MockResponse(base_schedule_response, 200)

    return MockResponse(None, 404)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['django_omise',
 'django_omise.admin',
 'django_omise.migrations',
 'django_omise.models',
 'django_omise.tests',
 'django_omise.tests.mockdata',
 'django_omise.utils']

package_data = \
{'': ['*'],
 'django_omise': ['locale/th/LC_MESSAGES/django.po',
                  'static/django_omise/css/*',
                  'static/django_omise/images/creditcardproviders/*',
                  'static/django_omise/js/*',
                  'templates/admin/django_omise/charge/*',
                  'templates/django_omise/*']}

install_requires = \
['omise>=0.11.0,<0.12.0']

entry_points = \
{'console_scripts': ['install_omise = '
                     'cli.dependencies:install_compatible_omise',
                     'test_coverage = cli.tests:run_test_coverage']}

setup_kwargs = {
    'name': 'django-omise',
    'version': '0.2.12',
    'description': 'Django models for Omise',
    'long_description': '> :warning: The latest pypi package for Omise has a metadata error. Please reinstall omise from this command:\n\n```\npip uninstall omise\npip install git+https://github.com/omise/omise-python@bfcf283378a823139b9f19f10e84d42a98c5b1ac\n```\n\n# django-omise Django + Omise\n\n![Test](https://github.com/jamesx00/django-omise/actions/workflows/tests.yml/badge.svg)\n[![Coverage Status](https://coveralls.io/repos/github/jamesx00/django-omise/badge.svg?branch=master)](https://coveralls.io/github/jamesx00/django-omise?branch=master)\n[![contributions welcome](https://img.shields.io/badge/contributions-welcome-brightgreen.svg?style=flat)](https://github.com/jamesx00/django-omise/issues)\n\nDjango models for Omise. Currently, we support the following features:\n\n-   Creating a customer\n-   Allowing customer to add/delete credit/debit cards\n-   Collect payments with new card with the option to keep the card\n-   Collect payments with saved cards\n-   Collect payments with [Internet Banking](https://www.omise.co/internet-banking)\n-   Collect payments with [TrueMoney Wallet](https://www.omise.co/truemoney-wallet)\n-   Collect payments with [Promptpay](https://www.omise.co/promptpay)\n-   Collect payments with [Rabbit LINE Pay](https://www.omise.co/rabbit-linepay)\n-   Webhook handler, saving raw data as an Event object and update related objects, currently supporting\n\n    -   Customer\n    -   Card\n    -   Charge\n    -   Source\n    -   Refund\n    -   Schedule\n    -   Scheduled Charge\n    -   Schedule Occurrence\n\n-   3DS pending charges handling\n\nSee the [roadmap](#roadmap-and-contributions) for the plan of this project. Contributions are welcome!\n\n### Quick start\n\n---\n\n1. Add "django_omise" to your INSTALLED_APPS setting like this:\n\n```\n    INSTALLED_APPS = [\n        ...\n        "django_omise",\n    ]\n```\n\n2. Include the django_omise URLconf in your project urls.py like this:\n\n```\n    path("payments/", include("django_omise.urls")),\n```\n\n3. Add Omise keys and operating mode in settings.py:\n\n```python\nOMISE_PUBLIC_KEY = xxxx\nOMISE_SECRET_KEY = xxxx\nOMISE_LIVE_MODE = True | False\nOMISE_CHARGE_RETURN_HOST = localhost:8000\n\n# Optional. The default payment method is credit/debit card only.\n# You must specify additional payment methods.\nOMISE_PAYMENT_METHODS = [\n    "card",\n    "internet_banking", # Internet Banking\n    "truemoney_wallet", # TrueMoney Wallet\n    "promptpay", # Promptpay\n    "rabbit_linepay", # Rabbit LINE Pay\n]\n```\n\n4. Run `python manage.py migrate` to create the Omise models.\n\n5. Add Omise endpoint webhook url `https://www.your-own-domain.com/payments/webhook/`\n\n### Basic usage\n\n---\n\n1. Create an Omise customer from User:\n\n    ```python\n    from django.contrib.auth import get_user_model\n    from django_omise.models.core import Customer\n\n    User = get_user_model()\n    user = User.objects.first()\n    customer = Customer.get_or_create(user=user)\n    ```\n\n2. Add card to Customer\n\n    2.1 With the built-in view (Recommended)\n\n    We have built a basic card collecting view where logged in users can add and remove their cards. Run Django server and visit _/payments/payment_methods/_ to see it in action. You could override the template used in the view by creating a new template in your project\'s directory _/templates/django_omise/manage_payment_methods.html_.\n\n    2.2 Manually\n\n    ```python\n    from django_omise.models.core import Customer\n    from django_omise.omise import omise\n\n    omise_token = omise.Token.retrieve(token_id)\n    Customer.objects.live().first().add_card(token=omise_token)\n    ```\n\n3. Charge a customer (Currently supporting new/saved cards, [Internet Banking](https://www.omise.co/internet-banking), [TrueMoney Wallet](https://www.omise.co/truemoney-wallet), [Promptpay](https://www.omise.co/promptpay))\n\n    3.1 With the build-in mixin\n\n    This package comes with a built-in mixin, with which you can create a class-based-view and write a few methods to charge a customer. See below for an example or see [Example 1](./examples/):\n\n    ```python\n    from django.contrib.auth.mixins import LoginRequiredMixin\n    from django_omise.mixins import CheckoutMixin\n    from django_omise.models.choices import Currency\n\n    # Your own class-based-view\n    class CheckoutView(LoginRequiredMixin, CheckoutMixin):\n\n        template_name = "yourapp/template.html"\n        success_url = ...\n\n        def get_charge_details(self):\n            return {\n                "amount": 100000,\n                "currency": Currency.THB,\n            }\n\n        def process_charge_and_form(self, charge, form):\n            if charge.status in [ChargeStatus.SUCCESSFUL, ChargeStatus.PENDING]:\n                # Create new order and attach a charge object\n                # And handle form data\n                handle_form_data(form.cleaned_data)\n    ```\n\n    3.2 Manually\n\n    ```python\n    from django_omise.models.choices import Currency, ChargeStatus\n    from django_omise.models.core import Customer\n\n    customer = Customer.objects.first()\n    card = customer.cards.live().first()\n\n    charge = customer.charge_with_card(\n        amount=100000,\n        currency=Currency.THB,\n        card=card,\n    )\n\n    if charge.status == ChargeStatus.SUCCESSFUL:\n        # Do something\n    elif charge.status == ChargeStatus.FAILED:\n        # Do something else\n    ```\n\n4. Create a charge schedule for a customer:\n\nAt the moment, you can create a new schedule for a customer manually by calling the method create_schedule from a Custoemr object. See below for an example:\n\n```python\nimport datetime\nfrom django_omise.models.choices import Currency\nfrom django_omise.models.core import Customer\n\ncustomer = Customer.objects.first()\ncard = customer.default_card\n\ncustomer.create_schedule(\n    amount=100000,\n    currency=Currency.THB,\n    card=card,\n    every=1,\n    period="month",\n    start_date=datetime.date(year=2022, month=5, day=22),\n    end_date=datetime.date(year=2032, month=5, day=22),\n    on={\n        \'days_of_month\': [22],\n    },\n    description="Monthly subscription",\n)\n```\n\n### Roadmap and contributions\n\n---\n\nHere are our immediate plans for this package, and more will be added! All contributions are welcome. I am new to publishing a public package, so if you have any recommendations, please feel free to create an issue on this repository or feel free to send me an email at siwatjames@gmail.com.\n\nOmise Features\n\n-   [x] Handle refunds API\n-   Handle webhook events and update related objects\n-   Create charge with Sources\n    -   [x] Internet banking\n    -   [x] TrueMoney Wallet\n    -   [x] Promptpay\n    -   [x] Rabbit LINE Pay\n    -   [ ] Installment\n-   Schedule\n    -   [x] Scheduled Charges\n    -   [ ] Scheduled Transfer\n\nOthers\n\n-   Implement tests\n-   Add documentations\n\n### Development\n\n---\n\nYou can run tests with either coverage or pytest.\n\nTo run with pytest\n\n```shell\npip install pytest-django\npython -m pytest [path_to_file] [--verbose -s --cov-report=html --cov=.]\n```\n\nTo run with coverage\n\n```shell\npip install coverage\ncoverage run run_tests.py\ncoverage report\ncoverage html\n```\n',
    'author': 'James Tansiri',
    'author_email': 'tansirijames@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/jamesx00/django-omise',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
